@@ -32,15 +32,17 @@
 #include "xfconf-gvaluefuncs.h"
 #include "xfconf-private.h"
 #include "xfconf-marshal.h"
+#include "xfconf-common-private.h"
 #if 0
 #include "xfconf-types.h"
-#include "xfconf-common-private.h"
 #include "xfconf.h"
 #include "xfconf-alias.h"
 #endif
 
+#if 0
 #define DEFAULT_MAX_ENTRIES  -1  /* no limit */
 #define DEFAULT_MAX_AGE      (60*60)  /* 1 hour */
+#endif
 
 #define ALIGN_VAL(val, align)  ( ((val) + ((align) -1)) & ~((align) - 1) )
 
@@ -55,21 +57,32 @@ static void xfconf_cache_mutex_unlock(GStaticMutex *mtx) { g_static_mutex_unlock
 
 typedef struct
 {
+#if 0
     GTimeVal last_used;
-    GValue value;
+#endif
+    GValue *value;
 } XfconfCacheItem;
 
 static XfconfCacheItem *
-xfconf_cache_item_new(const GValue *value)
+xfconf_cache_item_new(const GValue *value,
+                      gboolean steal)
 {
     XfconfCacheItem *item;
 
     g_return_val_if_fail(value, NULL);
 
     item = g_slice_new0(XfconfCacheItem);
+#if 0
     g_get_current_time(&item->last_used);
-    g_value_init(&item->value, G_VALUE_TYPE(value));
-    g_value_copy(value, &item->value);
+#endif
+
+    if(G_LIKELY(steal)) {
+        item->value = (GValue *) value;
+    } else {
+        item->value = g_new0(GValue, 1);
+        g_value_init(item->value, G_VALUE_TYPE(value));
+        g_value_copy(value, item->value);
+    }
 
     return item;
 }
@@ -78,15 +91,17 @@ static gboolean
 xfconf_cache_item_update(XfconfCacheItem *item,
                          const GValue *value)
 {
-    if(value && _xfconf_gvalue_is_equal(&item->value, value))
+    if(value && _xfconf_gvalue_is_equal(item->value, value))
         return FALSE;
 
+#if 0
     g_get_current_time(&item->last_used);
+#endif
 
     if(value) {
-        g_value_unset(&item->value);
-        g_value_init(&item->value, G_VALUE_TYPE(value));
-        g_value_copy(value, &item->value);
+        g_value_unset(item->value);
+        g_value_init(item->value, G_VALUE_TYPE(value));
+        g_value_copy(value, item->value);
 
         return TRUE;
     }
@@ -99,8 +114,8 @@ xfconf_cache_item_free(XfconfCacheItem *item)
 {
     g_return_if_fail(item);
 
-    g_value_unset(&item->value);
-
+    g_value_unset(item->value);
+    g_free(item->value);
     g_slice_free(XfconfCacheItem, item);
 }
 
@@ -184,8 +199,10 @@ struct _XfconfCache
 
     gchar *channel_name;
 
+#if 0
     gint max_entries;
     gint max_age;
+#endif
 
     GTree *properties;
 
@@ -215,8 +232,10 @@ enum
 {
     PROP0 = 0,
     PROP_CHANNEL_NAME,
+#if 0
     PROP_MAX_ENTRIES,
     PROP_MAX_AGE,
+#endif
 };
 
 static void xfconf_cache_set_g_property(GObject *object,
@@ -241,8 +260,6 @@ static void xfconf_cache_property_removed(DBusGProxy *proxy,
 
 
 static guint signals[N_SIGS] = { 0, };
-G_LOCK_DEFINE_STATIC(singletons);
-static GHashTable *singletons = NULL;
 
 
 G_DEFINE_TYPE(XfconfCache, xfconf_cache, G_TYPE_OBJECT)
@@ -257,7 +274,7 @@ xfconf_cache_class_init(XfconfCacheClass *klass)
     object_class->get_property = xfconf_cache_get_g_property;
     object_class->finalize = xfconf_cache_finalize;
 
-    signals[SIG_PROPERTY_CHANGED] = g_signal_new("property-changed",
+    signals[SIG_PROPERTY_CHANGED] = g_signal_new(I_("property-changed"),
                                                  XFCONF_TYPE_CACHE,
                                                  G_SIGNAL_RUN_LAST,
                                                  G_STRUCT_OFFSET(XfconfCacheClass,
@@ -280,7 +297,7 @@ xfconf_cache_class_init(XfconfCacheClass *klass)
                                                         | G_PARAM_STATIC_NAME
                                                         | G_PARAM_STATIC_NICK
                                                         | G_PARAM_STATIC_BLURB));
-
+#if 0
     g_object_class_install_property(object_class, PROP_MAX_ENTRIES,
                                     g_param_spec_int("max-entries",
                                                      "Maximum entries",
@@ -304,6 +321,7 @@ xfconf_cache_class_init(XfconfCacheClass *klass)
                                                      | G_PARAM_STATIC_NAME
                                                      | G_PARAM_STATIC_NICK
                                                      | G_PARAM_STATIC_BLURB));
+#endif
 }
 
 static void
@@ -344,7 +362,7 @@ xfconf_cache_set_g_property(GObject *object,
             g_free(cache->channel_name);
             cache->channel_name = g_value_dup_string(value);
             break;
-
+#if 0
         case PROP_MAX_ENTRIES:
             xfconf_cache_set_max_entries(cache, g_value_get_int(value));
             break;
@@ -352,7 +370,7 @@ xfconf_cache_set_g_property(GObject *object,
         case PROP_MAX_AGE:
             xfconf_cache_set_max_age(cache, g_value_get_int(value));
             break;
-
+#endif
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
             break;
@@ -371,7 +389,7 @@ xfconf_cache_get_g_property(GObject *object,
         case PROP_CHANNEL_NAME:
             g_value_set_string(value, cache->channel_name);
             break;
-
+#if 0
         case PROP_MAX_ENTRIES:
             g_value_set_int(value, cache->max_entries);
             break;
@@ -379,7 +397,7 @@ xfconf_cache_get_g_property(GObject *object,
         case PROP_MAX_AGE:
             g_value_set_int(value, cache->max_age);
             break;
-
+#endif
         default:
             G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
             break;
@@ -416,11 +434,6 @@ xfconf_cache_finalize(GObject *obj)
 
     g_static_mutex_free(&cache->cache_lock);
 
-    G_LOCK(singletons);
-    if(singletons)
-        g_hash_table_remove(singletons, cache);
-    G_UNLOCK(singletons);
-
     G_OBJECT_CLASS(xfconf_cache_parent_class)->finalize(obj);
 }
 
@@ -444,7 +457,7 @@ xfconf_cache_property_changed(DBusGProxy *proxy,
     if(item)
         changed = xfconf_cache_item_update(item, value);
     else {
-        item = xfconf_cache_item_new(value);
+        item = xfconf_cache_item_new(value, FALSE);
         g_tree_insert(cache->properties, g_strdup(property), item);
     }
 
@@ -519,7 +532,7 @@ xfconf_cache_set_property_reply_handler(DBusGProxy *proxy,
         g_error_free(error);
 
         if(old_item->item)
-            xfconf_cache_item_update(item, &old_item->item->value);
+            xfconf_cache_item_update(item, old_item->item->value);
         else {
             g_tree_remove(cache->properties, old_item->property);
             item = NULL;
@@ -530,7 +543,7 @@ xfconf_cache_set_property_reply_handler(DBusGProxy *proxy,
         g_signal_emit(G_OBJECT(cache), signals[SIG_PROPERTY_CHANGED],
                       g_quark_from_string(old_item->property),
                       cache->channel_name, old_item->property,
-                      item ? &item->value : &empty_val);
+                      item ? item->value : &empty_val);
         xfconf_cache_mutex_lock(&cache->cache_lock);
     }
 
@@ -577,66 +590,28 @@ out:
 }
 #endif
 
-static void
-xfconf_cache_destroyed(gpointer data,
-                       GObject *where_the_object_was)
-{
-    gchar *channel_name = data;
-
-    G_LOCK(singletons);
-    g_hash_table_remove(singletons, channel_name);
-    G_UNLOCK(singletons);
-}
-
 
 
 XfconfCache *
-xfconf_cache_get(const gchar *channel_name)
+xfconf_cache_new(const gchar *channel_name)
 {
-    XfconfCache *cache;
-
-    G_LOCK(singletons);
-
-    if(!singletons) {
-        singletons = g_hash_table_new_full(g_str_hash, g_str_equal,
-                                           (GDestroyNotify)g_free, NULL);
-    }
-
-    cache = g_hash_table_lookup(singletons, channel_name);
-    if(cache)
-        g_object_ref(G_OBJECT(cache));
-    else {
-        gchar *tmp = g_strdup(channel_name);
-
-        cache = g_object_new(XFCONF_TYPE_CACHE,
-                             "channel-name", channel_name,
-                             NULL);
-        g_hash_table_insert(singletons, tmp, cache);
-        g_object_weak_ref(G_OBJECT(cache), xfconf_cache_destroyed, tmp);
-    }
-
-    G_UNLOCK(singletons);
-
-    return cache;
+    return g_object_new(XFCONF_TYPE_CACHE,
+                        "channel-name", channel_name,
+                        NULL);
 }
 
-static void
+static gboolean
 xfconf_cache_prefetch_ht(gpointer key,
                          gpointer value,
                          gpointer user_data)
 {
-    gchar *property = key;
-    GValue *val = value;
     XfconfCache *cache = XFCONF_CACHE(user_data);
     XfconfCacheItem *item;
 
-    item = g_tree_lookup(cache->properties, property);
-    if(item)
-        xfconf_cache_item_update(item, val);
-    else {
-        item = xfconf_cache_item_new(val);
-        g_tree_insert(cache->properties, g_strdup(property), item);
-    }
+    item = xfconf_cache_item_new(value, TRUE);
+    g_tree_insert(cache->properties, key, item);
+
+    return TRUE;
 }
 
 gboolean
@@ -649,15 +624,16 @@ xfconf_cache_prefetch(XfconfCache *cache,
     DBusGProxy *proxy = _xfconf_get_dbus_g_proxy();
     GError *tmp_error = NULL;
 
+    g_return_val_if_fail(g_tree_nnodes(cache->properties) == 0, FALSE);
+
     xfconf_cache_mutex_lock(&cache->cache_lock);
 
     if(xfconf_client_get_all_properties(proxy, cache->channel_name,
                                         property_base ? property_base : "/",
                                         &props, &tmp_error))
     {
-        /* FIXME: perhaps change item API to allow 'stealing' a GValue rather
-         * than copying all the time */
-        g_hash_table_foreach(props, xfconf_cache_prefetch_ht, cache);
+        g_hash_table_foreach_steal(props, xfconf_cache_prefetch_ht, cache);
+        g_hash_table_destroy(props);
         /* TODO: honor max entries */
         ret = TRUE;
     } else
@@ -686,7 +662,7 @@ xfconf_cache_lookup_locked(XfconfCache *cache,
         if(xfconf_client_get_property(proxy, cache->channel_name,
                                       property, &tmpval, &tmp_error))
         {
-            item = xfconf_cache_item_new(&tmpval);
+            item = xfconf_cache_item_new(&tmpval, FALSE);
             g_tree_insert(cache->properties, g_strdup(property), item);
             g_value_unset(&tmpval);
             /* TODO: check tree for evictions */
@@ -697,17 +673,19 @@ xfconf_cache_lookup_locked(XfconfCache *cache,
     if(item) {
         if(value) {
             if(!G_VALUE_TYPE(value))
-                g_value_init(value, G_VALUE_TYPE(&item->value));
+                g_value_init(value, G_VALUE_TYPE(item->value));
 
-            if(G_VALUE_TYPE(value) == G_VALUE_TYPE(&item->value))
-                g_value_copy(&item->value, value);
+            if(G_VALUE_TYPE(value) == G_VALUE_TYPE(item->value))
+                g_value_copy(item->value, value);
             else {
-                if(!g_value_transform(&item->value, value))
+                if(!g_value_transform(item->value, value))
                     item = NULL;
             }
         }
+#if 0
         if(item)
             xfconf_cache_item_update(item, NULL);
+#endif
     }
 
     return !!item;
@@ -786,7 +764,7 @@ xfconf_cache_set(XfconfCache *cache,
 
     if(item) {
         /* if the value isn't changing, there's no reason to continue */
-        if(_xfconf_gvalue_is_equal(&item->value, value)) {
+        if(_xfconf_gvalue_is_equal(item->value, value)) {
             xfconf_cache_mutex_unlock(&cache->cache_lock);
             return TRUE;
         }
@@ -808,7 +786,7 @@ xfconf_cache_set(XfconfCache *cache,
     } else {
         old_item = xfconf_cache_old_item_new(property);
         if(item)
-            old_item->item = xfconf_cache_item_new(&item->value);
+            old_item->item = xfconf_cache_item_new(item->value, FALSE);
         g_hash_table_insert(cache->old_properties, old_item->property, old_item);
     }
 
@@ -826,7 +804,7 @@ xfconf_cache_set(XfconfCache *cache,
     if(item)
         xfconf_cache_item_update(item, value);
     else {
-        item = xfconf_cache_item_new(value);
+        item = xfconf_cache_item_new(value, FALSE);
         g_tree_insert(cache->properties, g_strdup(property), item);
     }
 
@@ -940,6 +918,7 @@ xfconf_cache_reset(XfconfCache *cache,
     return ret;
 }
 
+#if 0
 void
 xfconf_cache_set_max_entries(XfconfCache *cache,
                              gint max_entries)
@@ -971,3 +950,4 @@ xfconf_cache_get_max_age(XfconfCache *cache)
 {
     return cache->max_age;
 }
+#endif
