@@ -217,7 +217,7 @@ xfconf_channel_constructor(GType type,
     const gchar *channel_name = NULL;
     gboolean is_singleton = IS_SINGLETON_DEFAULT;
     guint i;
-    XfconfChannel *channel;
+    XfconfChannel *channel = NULL;
 
     for(i = 0; i < n_construct_properties; ++i) {
         if(!strcmp(g_param_spec_get_name(construct_properties[i].pspec), "channel-name"))
@@ -238,9 +238,9 @@ xfconf_channel_constructor(GType type,
             __channel_singletons = g_hash_table_new_full(g_str_hash, g_str_equal,
                                                          (GDestroyNotify)g_free,
                                                          (GDestroyNotify)g_object_unref);
-        }
+        } else
+            channel = g_hash_table_lookup(__channel_singletons, channel_name);
 
-        channel = g_hash_table_lookup(__channel_singletons, channel_name);
         if(!channel) {
             channel = XFCONF_CHANNEL(G_OBJECT_CLASS(xfconf_channel_parent_class)->constructor(type,
                                                                                               n_construct_properties,
@@ -257,7 +257,7 @@ xfconf_channel_constructor(GType type,
     }
 
     if(!channel->cache) {
-        channel->cache = xfconf_cache_get(channel_name);
+        channel->cache = xfconf_cache_new(channel_name);
         xfconf_cache_prefetch(channel->cache, channel->property_base, NULL);
         g_signal_connect(channel->cache, "property-changed",
                          G_CALLBACK(xfconf_channel_property_changed), channel);
@@ -574,6 +574,10 @@ xfconf_channel_get(const gchar *channel_name)
  * lifetime (and thus the lifetime of connected signals and bound
  * #GObject properties) to the lifetime of another object.
  *
+ * Also note that each channel has its own cache, so if you create
+ * 2 new channels with the same name, it will double the dbus traffic,
+ * so in this cases it is highly recommended to use xfconf_channel_get().
+ *
  * Returns: A new #XfconfChannel.  Release with g_object_unref()
  *          when no longer needed.
  **/
